# coding=utf-8

import urllib.parse
from io import BytesIO

from django.db import transaction
from django.http import HttpResponse
from rest_framework.views import APIView
from utils.permission import IsAdministrator
from utils import response_error, response_ok
from utils.custom_modelviewset import CustomModelViewSet
from apps.system.models import SysLog

from apps.examination.exampaper.models import ExamPaper
from apps.examination.exampaper.filters import ExamPaperFilter
from apps.examination.exampaper.serializers import ExamPaperSerializer


class ExamPaperViewSet(CustomModelViewSet):
    permission_classes = [IsAdministrator, ]
    queryset = ExamPaper.objects.filter(delete=False)
    serializer_class = ExamPaperSerializer

    def filter_queryset(self, queryset):
        f = ExamPaperFilter(self.request.GET, queryset=queryset)
        return f.qs

    def perform_create(self, serializer):
        super(ExamPaperViewSet, self).perform_create(serializer)
        instance = serializer.instance
        validated_data = serializer.validated_data
        SysLog.objects.addnew(self.request.user, SysLog.INSERT, u'添加试卷[%s]，id=%d' % (instance.name, instance.id), validated_data)

    def perform_update(self, serializer):
        super(ExamPaperViewSet, self).perform_update(serializer)
        instance = serializer.instance
        validated_data = serializer.validated_data
        SysLog.objects.addnew(self.request.user, SysLog.UPDATE, u'修改试卷[%s]，id=%d' % (instance.name, instance.id), validated_data)

    def destroy(self, request, *args, **kwargs):
        with transaction.atomic():
            instance = self.get_object()
            instance.delete = True
            instance.save()
            SysLog.objects.addnew(self.request.user, SysLog.DELETE, u'删除试卷[%s]，id=%d' % (instance.name, instance.id))
        return response_ok()
    
    def export(self, request, *args, **kwargs):
        instance = self.get_object()
        doc = instance.create_docx()

        buffer = BytesIO()
        doc.save(buffer)
        buffer.seek(0)

        type = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
        filename = f'{instance.name}试卷.docx'
        encoded_filename = urllib.parse.quote(filename)
        content_disposition = f'attachment; filename="{encoded_filename}"'

        response = HttpResponse(buffer, content_type=type)
        response['Content-Disposition'] = content_disposition
        return response